//
//  SPK_XYCalibratorPlugIn.m
//  SPK-XYCalibrator
//
//  Created by Toby Harris on 21/02/2008.
//  Copyright (c) 2008 *spark audio-visual. Some rights reserved.
//

/* It's highly recommended to use CGL macros instead of changing the current context for plug-ins that perform OpenGL rendering */
#import <OpenGL/CGLMacro.h>

#import "SPK_XYCalibratorPlugIn.h"

#define	kQCPlugIn_Name				@"SPK-XYCalibrator"
#define	kQCPlugIn_Description		@"Scales the XY input to the rendering context's bounds, based upon max extents set during calibration mode"

@implementation SPK_XYCalibratorPlugIn

/*
Here you need to declare the input / output properties as dynamic as Quartz Composer will handle their implementation
@dynamic inputFoo, outputBar;
*/

@dynamic inputX, inputY, outputX, outputY, inputCalibrateMode;
@synthesize scaleX, scaleY;

+ (NSDictionary*) attributes
{
	/*
	Return a dictionary of attributes describing the plug-in (QCPlugInAttributeNameKey, QCPlugInAttributeDescriptionKey...).
	*/
	
	return [NSDictionary dictionaryWithObjectsAndKeys:kQCPlugIn_Name, QCPlugInAttributeNameKey, kQCPlugIn_Description, QCPlugInAttributeDescriptionKey, nil];
}

+ (NSDictionary*) attributesForPropertyPortWithKey:(NSString*)key
{
	/*
	Specify the optional attributes for property based ports (QCPortAttributeNameKey, QCPortAttributeDefaultValueKey...).
	*/
	
	if([key isEqualToString:@"inputX"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
				@"X", QCPortAttributeNameKey,
				[NSNumber numberWithDouble:0], QCPortAttributeDefaultValueKey,
				nil];
	if([key isEqualToString:@"inputY"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
				@"Y", QCPortAttributeNameKey,
				[NSNumber numberWithDouble:0], QCPortAttributeDefaultValueKey,
				nil];
	if([key isEqualToString:@"inputCalibrateMode"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
				@"Calibrate", QCPortAttributeNameKey,
				nil];
	if([key isEqualToString:@"outputX"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
				@"X", QCPortAttributeNameKey,
				nil];
	if([key isEqualToString:@"outputY"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
				@"Y", QCPortAttributeNameKey,
				nil];
	return nil;
}

+ (NSArray*) sortedPropertyPortKeys
{
	/*
	 Override this method to specify an optional ordering for property based ports in the user interface.
	 */
	return [NSArray arrayWithObjects:@"inputCalibrateMode",@"inputX",@"inputY",@"outputX",@"outputY",nil];
}

+ (QCPlugInExecutionMode) executionMode
{
	/*
	Return the execution mode of the plug-in: kQCPlugInExecutionModeProvider, kQCPlugInExecutionModeProcessor, or kQCPlugInExecutionModeConsumer.
	*/
	
	return kQCPlugInExecutionModeProcessor;
}

+ (QCPlugInTimeMode) timeMode
{
	/*
	Return the time dependency mode of the plug-in: kQCPlugInTimeModeNone, kQCPlugInTimeModeIdle or kQCPlugInTimeModeTimeBase.
	*/
	
	return kQCPlugInTimeModeNone;
}

+ (NSArray*) plugInKeys
{
	/*
	 Return a list of the KVC keys corresponding to the internal settings of the plug-in.
	 */
	return [NSArray arrayWithObjects:@"scaleX",@"scaleY",nil];
}

- (id) init
{
	if(self = [super init]) {
		/*
		Allocate any permanent resource required by the plug-in.
		*/
	}
	
	return self;
}

- (void) finalize
{
	/*
	Release any non garbage collected resources created in -init.
	*/
	
	[super finalize];
}

- (void) dealloc
{
	/*
	Release any resources created in -init.
	*/
	
	[super dealloc];
}

//// Self Methods

-(void)resetCalibrationVars
{
	maxX = 0.1;
	maxY = 0.1;
}

@end

@implementation SPK_XYCalibratorPlugIn (Execution)

- (BOOL) startExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when rendering of the composition starts: perform any required setup for the plug-in.
	Return NO in case of fatal failure (this will prevent rendering of the composition to start).
	*/
	
	[self resetCalibrationVars];
	
	return YES;
}

- (void) enableExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when the plug-in instance starts being used by Quartz Composer.
	*/
}

- (BOOL) execute:(id<QCPlugInContext>)context atTime:(NSTimeInterval)time withArguments:(NSDictionary*)arguments
{
	/*
	Called by Quartz Composer whenever the plug-in instance needs to execute.
	Only read from the plug-in inputs and produce a result (by writing to the plug-in outputs or rendering to the destination OpenGL context) within that method and nowhere else.
	Return NO in case of failure during the execution (this will prevent rendering of the current frame to complete).
	
	The OpenGL context for rendering can be accessed and defined for CGL macros using:
	CGLContextObj cgl_ctx = [context CGLContextObj];
	*/
	
	if (self.inputCalibrateMode) {
		// reset maximums etc on gate going high
		if (calibrationModeReset) {
			[self resetCalibrationVars];
			calibrationModeReset = NO;
		}
		
		// find extends of input, ie out of kineme wiimote i tend to see +-0.7
		//maxX = max(maxX,self.inputX);
		//maxY = max(maxY,self.inputY);
		maxX = (maxX > self.inputX) ? maxX : self.inputX;
		maxY = (maxY > self.inputY) ? maxY : self.inputY;
		
		
		// outputX = inputX * scaleX; rendering context width / 2 = maxX * scaleX
		self.scaleX = [context bounds].size.width / (2 * maxX);
		self.scaleY = [context bounds].size.height / (2 * maxY);
		
		[context logMessage:@"scaleX %f",self.scaleX];
		
	} else {
		// set up signal vs gate
		calibrationModeReset = YES;
	}
	
	self.outputX = self.inputX * self.scaleX;
	self.outputY = self.inputY * self.scaleY;
	
	return YES;
}

- (void) disableExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when the plug-in instance stops being used by Quartz Composer.
	*/
}

- (void) stopExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when rendering of the composition stops: perform any required cleanup for the plug-in.
	*/
}

@end
